@@ -47,6 +47,7 @@ my $g_flush_tables = 2;
 my $g_orig_master_is_new_slave;
 my $g_running_updates_limit = 1;
 my $g_running_seconds_limit = 10;
+my $g_seconds_behind_master = 30;
 my $g_skip_lock_all_tables;
 my $g_remove_orig_master_conf;
 my $g_interactive = 1;
@@ -63,12 +64,12 @@ sub identify_orig_master() {
   $log->info();
   $log->info("* Phase 1: Configuration Check Phase..\n");
   $log->info();
-  my @servers_config = new MHA::Config(
+  my ( $sc_ref, undef ) = new MHA::Config(
     logger     => $log,
     globalfile => $g_global_config_file,
     file       => $g_config_file
   )->read_config();
-
+  my @servers_config = @$sc_ref;
   $log = MHA::ManagerUtil::init_log( undef, $servers_config[0]->{log_level} );
 
   unless ($g_workdir) {
@@ -101,7 +102,7 @@ sub identify_orig_master() {
   $orig_master = $_server_manager->get_current_alive_master();
   if ( !$orig_master ) {
     $log->error(
-"Failed to get currnet master. Maybe replication is misconfigured or configuration file is broken?"
+"Failed to get current master. Maybe replication is misconfigured or configuration file is broken?"
     );
     croak;
   }
@@ -153,7 +154,7 @@ sub identify_orig_master() {
     }
   }
 
-  $_server_manager->check_replication_health($g_running_updates_limit);
+  $_server_manager->check_replication_health($g_seconds_behind_master);
 
   my @threads =
     $orig_master->get_running_update_threads( $g_running_updates_limit + 1 );
@@ -284,14 +285,23 @@ sub reject_update($$) {
   $log->info();
   if ( $new_master->{master_ip_online_change_script} ) {
     my $command =
-"$orig_master->{master_ip_online_change_script} --command=stop --orig_master_host=$orig_master->{hostname} --orig_master_ip=$orig_master->{ip} --orig_master_port=$orig_master->{port} --orig_master_user=$orig_master->{escaped_user} --orig_master_password=$orig_master->{escaped_password} --new_master_host=$new_master->{hostname} --new_master_ip=$new_master->{ip} --new_master_port=$new_master->{port} --new_master_user=$new_master->{escaped_user} --new_master_password=$new_master->{escaped_password}";
+"$orig_master->{master_ip_online_change_script} --command=stop --orig_master_host=$orig_master->{hostname} --orig_master_ip=$orig_master->{ip} --orig_master_port=$orig_master->{port} --orig_master_user=$orig_master->{escaped_user} --new_master_host=$new_master->{hostname} --new_master_ip=$new_master->{ip} --new_master_port=$new_master->{port} --new_master_user=$new_master->{escaped_user}";
+    $command .= " --orig_master_ssh_user=$orig_master->{ssh_user}";
+    $command .= " --new_master_ssh_user=$new_master->{ssh_user}";
     $command .= $orig_master->get_ssh_args_if( 1, "orig", 1 );
     $command .= $new_master->get_ssh_args_if( 2, "new", 1 );
+    if ($g_orig_master_is_new_slave) {
+      $command .= " --orig_master_is_new_slave";
+    }
     $log->info(
 "Executing master ip online change script to disable write on the current master:"
     );
-    $log->info("  $command");
+    $log->info(
+      "  $command --orig_master_password=xxx --new_master_password=xxx");
+    $command .=
+" --orig_master_password=$orig_master->{escaped_password} --new_master_password=$new_master->{escaped_password}";
     my ( $high, $low ) = MHA::ManagerUtil::exec_system($command);
+
     if ( $high == 0 && $low == 0 ) {
       $log->info(" ok.");
     }
@@ -393,14 +403,23 @@ sub switch_master($$$$) {
   # Allow write access on the new master
   if ( $new_master->{master_ip_online_change_script} ) {
     my $command =
-"$new_master->{master_ip_online_change_script} --command=start --orig_master_host=$orig_master->{hostname} --orig_master_ip=$orig_master->{ip} --orig_master_port=$orig_master->{port} --orig_master_user=$orig_master->{escaped_user} --orig_master_password=$orig_master->{escaped_password} --new_master_host=$new_master->{hostname} --new_master_ip=$new_master->{ip} --new_master_port=$new_master->{port} --new_master_user=$new_master->{escaped_user} --new_master_password=$new_master->{escaped_password}";
+"$new_master->{master_ip_online_change_script} --command=start --orig_master_host=$orig_master->{hostname} --orig_master_ip=$orig_master->{ip} --orig_master_port=$orig_master->{port} --orig_master_user=$orig_master->{escaped_user} --new_master_host=$new_master->{hostname} --new_master_ip=$new_master->{ip} --new_master_port=$new_master->{port} --new_master_user=$new_master->{escaped_user}";
+    $command .= " --orig_master_ssh_user=$orig_master->{ssh_user}";
+    $command .= " --new_master_ssh_user=$new_master->{ssh_user}";
     $command .= $orig_master->get_ssh_args_if( 1, "orig", 1 );
     $command .= $new_master->get_ssh_args_if( 2, "new", 1 );
+    if ($g_orig_master_is_new_slave) {
+      $command .= " --orig_master_is_new_slave";
+    }
     $log->info(
 "Executing master ip online change script to allow write on the new master:"
     );
-    $log->info("  $command");
+    $log->info(
+      "  $command --orig_master_password=xxx --new_master_password=xxx");
+    $command .=
+" --orig_master_password=$orig_master->{escaped_password} --new_master_password=$new_master->{escaped_password}";
     my ( $high, $low ) = MHA::ManagerUtil::exec_system($command);
+
     if ( $high == 0 && $low == 0 ) {
       $log->info(" ok.");
     }
@@ -552,6 +571,10 @@ sub switch_slaves($$$$$$) {
 
   if ($g_orig_master_is_new_slave) {
     $log->info("Starting orig master as a new slave..");
+    if ( exists( $new_master->{use_ip_for_change_master} ) ) {
+      $orig_master->{use_ip_for_change_master} =
+        $new_master->{use_ip_for_change_master};
+    }
     if (
       $_server_manager->change_master_and_start_slave(
         $orig_master, $new_master, $master_log_file, $master_log_pos
@@ -696,6 +719,7 @@ sub main {
     'orig_master_is_new_slave' => \$g_orig_master_is_new_slave,
     'running_updates_limit=i'  => \$g_running_updates_limit,
     'running_seconds_limit=i'  => \$g_running_seconds_limit,
+    'seconds_behind_master=i'  => \$g_seconds_behind_master,
     'skip_lock_all_tables'     => \$g_skip_lock_all_tables,
     'remove_dead_master_conf'  => \$g_remove_orig_master_conf,
     'remove_orig_master_conf'  => \$g_remove_orig_master_conf,
