@@ -7,7 +7,13 @@ sleep 1
 mysql $S1 -e "set global read_only=1"
 
 mysql $S2 -e "stop slave"
+is_gtid_supported
+if test $? = 1
+then
+mysql $S2 -e "change master to master_host='127.0.0.1', master_port=$S1P, master_user='rsandbox', master_password='rsandbox'"
+else
 mysql $S2 -e "change master to master_host='127.0.0.1', master_port=$S1P, master_user='rsandbox', master_password='rsandbox', master_log_file='mysql-bin.000001', master_log_pos=4"
+fi
 mysql $S2 -e "start slave"
 
 check_master $0 $S2P $S1P
