@@ -1,6 +1,12 @@
 . ./init.sh
 
+is_gtid_supported
+if test $? = 1
+then
+mysql $M -e "change master to master_host='127.0.0.1', master_port=$S1P, master_user='rsandbox', master_password='rsandbox', master_auto_position=1; start slave"
+else
 mysql $M -e "change master to master_host='127.0.0.1', master_port=$S1P, master_user='rsandbox', master_password='rsandbox', master_log_file='mysql-bin.000001', master_log_pos=4; start slave"
+fi
 
 mysql $M test -e "insert into t1 values(2, 200, 'aaaaaa')"
 
