@@ -7,6 +7,11 @@ sleep 1
 ./start_m.sh --log-slave-updates
 ./stop_s1.sh
 ./start_s1.sh --log-slave-updates
+is_gtid_supported
+if test $? = 0
+then
+mysql $S1 -e "reset master"
+fi
 mysql $S1 -e "stop slave io_thread;start slave io_thread"
 mysql $S2 -e "stop slave io_thread;start slave io_thread"
 mysql $S3 -e "stop slave io_thread;start slave io_thread"
@@ -14,7 +19,13 @@ mysql $S4 -e "stop slave io_thread;start slave io_thread"
 sleep 5
 
 mysql $S2 -e "stop slave"
+is_gtid_supported
+if test $? = 1
+then
+mysql $S2 -e "change master to master_host='127.0.0.1', master_port=$S1P, master_user='rsandbox', master_password='rsandbox', master_auto_position=1"
+else
 mysql $S2 -e "change master to master_host='127.0.0.1', master_port=$S1P, master_user='rsandbox', master_password='rsandbox', master_log_file='mysql-bin.000001', master_log_pos=4"
+fi
 mysql $S2 -e "start slave"
 
 check_master $0 $S2P $S1P
