@@ -1,5 +1,8 @@
 . ./init.sh
+skip_if_gtid $0
 
+mysql $M test -e "insert into t1 values (10, 100, 100)"
+mysql $M test -e "insert into t1 values (11, 100, 100)"
 # S1 SQL thread stops with error
 mysql $S1 test -e "insert into t1 values (100, 100, 100)"
 mysql $M test -e "insert into t1 values (100, 100, 100)"
@@ -28,6 +31,6 @@ fail_if_nonzero $0 $?
 check_sql_yes $0 $S2P
 
 mysql $S1 test -e "insert into t1 values(10000003, 300, 'bbbaaaaaaa');"
-./check $0 3
+./check $0 5
 
 
